
import numpy as np
import pandas as pd


ROUND_NB_DIGIT = 2


def vol_unit(unit):
    """
    to help express vol either in bp/day or bp/year
    """
    u = unit.upper()
    if u == 'DAY' or u == 'D':
        return 1 / 252**0.5
    else:
        return 1


def get_diff(data, start_date, end_date):
    """
    get difference between 2 dates
    """
    mask_1 = (data.index.get_level_values('date') == end_date)
    mask_2 = (data.index.get_level_values('date') == start_date)
    one = data.loc[mask_1]
    one.index = one.index.droplevel('date')
    two = data.loc[mask_2]
    two.index = two.index.droplevel('date')
    # res=res.index.droplevel('date')
    return one - two


def get_max(data, resize=False, *args):
    """
    get max
    TBC
    """
    if resize:
        data = select_data(data, *args)
    x = data.groupby(level='expiry').apply(max)
    return x.reindex(data.index.get_level_values('expiry').unique(), axis=0)


def get_min(data, resize=False, *args):
    """
    get min
    TBC
    """
    if resize:
        data = select_data(data, *args)
    x = data.groupby(level='expiry').apply(min)
    return x.reindex(data.index.get_level_values('expiry').unique(), axis=0)


def get_surface(data, date):
    """
    get surface for given date
    TBC
    """
    mask = (data.index.get_level_values('date') == date)
    x = data.loc[mask]
    return drop_level(x, 'date')


def select_data(data, args):
    """
    select data from dataframe
    args:
        for discrete values
            field, value1, value2, etc
        for range of values
            field, value_start, value_end, 'range'

    returns subset dataframe
    TBC
    """
    arg = args
    if arg[0] == 'date':
        if len(arg) == 4 and arg[3] == 'range':
            mask = (data.index.get_level_values(arg[0]) >= arg[1]) & (
                data.index.get_level_values(arg[0]) <= arg[2])
        else:
            mask = (data.index.get_level_values(arg[0]) == arg[1])
            for ar in arg[2:len(arg)]:
                mask = mask | (data.index.get_level_values(arg[0]) == ar)
    else:
        mask = (data.index.get_level_values(arg[0]) == arg[1])
        for ar in arg[2:len(arg)]:
            mask = mask | (data.index.get_level_values(arg[0]) == ar)

    return data.loc[mask]


def drop_level(data, index):
    """
    drop level of input dataframe - inplace
    TBC
    """
    data.index = data.index.droplevel(index)
    return data


def get_percentile(data, date, resize=False, *args):
    """
    get percentile rank from input dataframe
    TBC
    """
    if resize:
        data = select_data(data, *args)
    return get_surface(data.groupby(level='expiry').rank(pct=True), date)


def get_mean(data, resize=False, *args):
    """
    get mean
    TBC
    """
    if resize:
        data = select_data(data, *args)
    data = data.dropna()
    x = data.groupby(level='expiry').apply(np.mean)
    return x.reindex(data.index.get_level_values('expiry').unique(), axis=0)


def get_std(data, resize=False, *args):
    """
    get standard deviation
    TBC
    """
    if resize:
        data = select_data(data, *args)
    data = data.dropna()
    x = data.groupby(level='expiry').apply(np.std)
    return x.reindex(data.index.get_level_values('expiry').unique(), axis=0)


def get_z_score(data, end_dt, resize=False, *args):
    """
    get z-score
    TBC
    """
    if resize:
        data = select_data(data, *args)
    return (get_surface(data, end_dt) - get_mean(data)) / get_std(data)


def build_weight(spread, weight, **kwargs):
    """
    build weight
    TBC
    """
    if (len(spread) is not len(weight)):
        if kwargs == {}:
            kwargs.setdefault('type', 'S')
        else:
            kwargs = kwargs['kwargs']

        if (len(spread) % 2 == 0 or (len(spread) % 2 == 0 and len(spread) % 3 == 0)) and (kwargs['type'].upper() != 'FLY' or kwargs['type'].upper() != 'F'):
            print('spread - weight len mismatch by default spread weighted (-1,1)')
            w = np.repeat(1, len(spread))
            pos = np.arange(1, len(w), 2)
            w[pos] = w[pos] * -1
        elif (len(spread) % 3 == 0 or (len(spread) % 2 == 0 and len(spread) % 3 == 0)) and (kwargs['type'].upper() == 'FLY' or kwargs['type'].upper() == 'F'):
            print('fly - weight len mismatch by default fly weighted (-1,2,-1)')
            w = np.repeat(-1, len(spread))
            pos = np.arange(1, len(w), 2)
            w[pos] = w[pos] * -2
    else:
        w = weight
    return w

# def build_spread(data, spread, typ_, resize=False, *args):
#     """
#     build spread
#     TBC
#     """
#     if resize:
#         data = select_data(data, args)
#     res = pd.DataFrame()
#     if typ_.upper() == "S" or typ_.upper() == "SLOPE":
#         for s in spread:
#             res[s[0] + s[1]] = data[s[1]] - data[s[0]]
#         return res
#     if typ_.upper() == "C" or typ._upper() == "CALENDAR":
#         from collections import OrderedDict
#         dic = OrderedDict()
#         for s in spread:
#             dic[(s[0] + s[1])] = data.loc[s[1]] - data.loc[s[0]]
#         res = pd.concat(dic.values(), keys=dic.keys())
#         res.index.names = ('expiry', 'date')
#         return res


def build_spread(data, spread, axis="T", weight=[], resize=False, *args, **kwargs):
    """
    build weight
    TBC
    """
    if resize:
        data = select_data(data, args)
    res = pd.DataFrame()
    w = build_weight(spread[0], weight, **kwargs)
    if axis.upper() == 'T' or axis.upper() == 'TENOR':
        for s in spread:
            nme = str()
            for i, e in enumerate(s):
                if i == 0:
                    res_tmp = data[s[i]] * w[i]
                else:
                    res_tmp = res_tmp + data[s[i]] * w[i]
                nme = nme + s[i]
            res[nme] = res_tmp
        return res

    if axis.upper() == 'E' or axis.upper() == 'EXPIRY':
        from collections import OrderedDict
        dic = OrderedDict()
        idx = data.loc[spread[0][0]].index
        col = data.loc[spread[0][0]].columns
        for s in spread:
            res_tmp = pd.DataFrame(0, index=idx, columns=col)
            nme = str()
            for i, e in enumerate(s):
                res_tmp = res_tmp + data.loc[s[i]] * w[i]
                nme = nme + s[i]
            dic[nme] = res_tmp
        res = pd.concat(dic.values(), keys=dic.keys())
        res.index.names = ('expiry', 'date')
        return res




class ObjFromDict:
    """
    Returns object with same structure as dict
    """

    def __init__(self, dic):
        """
        """
        for k, v in dic.items():
            if isinstance(v, dict):
                self.__dict__[k] = ObjFromDict(v)
            else:
                self.__dict__[k] = v

    def __repr__(self):
        return str(self.__dict__)

    def __copy__(self):
        return ObjFromDict(self.__dict__)




import copy

import pandas as pd

from IPython.display import Markdown

from ._util import Util
from sgmarkets_api_auth.util import save_result


class ResponseRotbComputeStrategyPrices:
    """
    TBU
    """

    def __init__(self,
                 li_raw_data=None,
                 obj_req=None):
        """
        """
        msg1 = 'Error: li_raw_data must be a list - Run call_api() again with debug=True'
        msg2 = 'Error: Each dic_res must be a list - Run call_api() again with debug=True'
        msg3 = 'Error: stratedySerie must be a key of each dic_res - Run call_api() again with debug=True'

        assert isinstance(li_raw_data, list), msg1
        for dic_res in li_raw_data:
            assert isinstance(dic_res, dict), msg2
            assert 'strategySerie' in dic_res, msg3

        raw_data = []
        for dic_res in li_raw_data:
            raw_data += dic_res['strategySerie']

        self.raw_data = copy.deepcopy(raw_data)
        self.obj_req = obj_req

        self.df_req, self.df_res = self._build_df_res_req()
        self.dic_req_param, self.dic_res_param = self._build_dic_param()

    def _get_dates(self, df_res):
        """
        """
        dic = self.obj_req.df_top.to_dict()
        dic = dic['Value']

        if 'dates' in dic:
            res = dic['dates'].replace("'", '"')
            return json.loads(res)
            
        return Util.get_unique_list(df_res['date'])

    def _build_df_res_req(self):
        """
        """
        # df_res (response)
        li_data = [f for f in self.raw_data]

        for e in li_data:
            if 'greeks' in e:
                for greek in ['delta', 'gamma', 'vega', 'theta']:
                    e[greek] = e['greeks'][greek]
                e.pop('greeks')
        df_res = pd.DataFrame(li_data)

        # build list of dates
        li_date = self._get_dates(df_res)
        N = len(li_date)

        # df_req (request)
        # duplicate df_leg by number of dates
        df_leg = self.obj_req.df_leg
        df_req = pd.concat([df_leg] * N,
                           axis=0).reset_index(drop=True)

        # reorder results by date then initial order (tag)
        df_res['tag'] = range(len(df_res))
        df_res = df_res.sort_values(['date', 'tag']).reset_index(drop=True)

        # move date from df_res to df_req (more natural)
        df_req['date'] = pd.to_datetime(df_res['date'].copy())
        df_res = df_res.drop('date', axis=1)

        # join df_req and df_res to make df_set
        df_res.index = df_req["date"].iloc[0:N]

        return df_req, df_res

    def _build_dic_param(self):
        """
        """
        dic_req = self.df_req.to_dict()
        dic_req_param = {k: Util.get_unique_list(v.values())
                         for k, v in dic_req.items()}

        dic_data = self.df_res.to_dict()
        dic_res_param = {k: Util.get_unique_list(v.values())
                         for k, v in dic_data.items()}

        return dic_req_param, dic_res_param

    def save(self,
             folder_save='dump',
             name=None,
             tagged=True,
             excel=False):
        """ 
        Split save for strategyprice type since no df.set in the object
        """
        if name is None:
            name = 'SG_Research_ROTB'

        save_result(self.df_res,
                    folder_save,
                    name=name + 'Strategy_res',
                    tagged=tagged,
                    excel=excel)

        save_result(self.df_req,
                    folder_save,
                    name=name + 'Strategy_req',
                    tagged=tagged,
                    excel=excel)

    def _repr_html_(self):
        """
        """
        return self.df_res.to_html()

    def info(self):
        """
        """
        md = """
A PostprocessROTB object from ComputeStrategyPrices endpoint has the properties:
+ `df_req`: request data (dataframe)
+ `df_res`: response data (dataframe)


+ `dic_req_param`: params in request, each param contains a list of all values taken (dictionary)
+ `dic_res_param`: params in response, each param contains a list of all values taken (dictionary)

+ `raw_data`: raw data in response under key 'componentSeries' (dictionary)

and the methods:
+ `save()` to save the data as `.csv` and `.xlsx` files
        """
        return Markdown(md)




from timeit import default_timer as timer
from IPython.display import Markdown

from ._util import Util
from .response_rotb_curves import ResponseRotbCurves


class RequestRotbCurves:
    """
    """

    def __init__(self):
        """
        """
        dic_url = {'base_url': 'https://analytics-api.sgmarkets.com',
                   'service': '/rotb',
                   'endpoint': '/v1/curves'}
        self.url = Util.build_url(dic_url)

    def info(self):
        """
        """
        md = """
A RequestRotbCurves object has the properties:
+ `url`: {}

and the methods:
+ `call_api()` to make the request to the url
        """.format(self.url)
        return Markdown(md)

    def call_api(self,
                 api,
                 debug=False):
        """
        See https://analytics-api.sgmarkets.com/rotb/swagger/ui/index#!/Curves/Curves_Curves
        """
        t0 = timer()
        print('calling API...')
        raw_response = api.get(self.url)
        t1 = timer()
        print('done in {:.2f} s'.format(t1 - t0))
        if debug:
            print('*** START DEBUG ***\n{}\n*** END DEBUG ***'.format(raw_response))
        response = ResponseRotbCurves(raw_response)
        return response




from copy import deepcopy

import datetime as dt
import pandas as pd

from sgmarkets_api_auth.util import save_result


class SliceRotbComputeStrategy():
    """
    TBU
    """

    def __init__(self,
                 obj_res,
                 x=None,
                 y=None,
                 z=None,
                 dic_req_fix={},
                 value=None,
                 y_pos='index',  # index or column
                 other_pos='column'):
        """
        """

        msg1 = 'Error: {} must be a column name in components.df_set'
        msg2 = 'Error: {} must be a value in column {} of components.df_set'
        msg3 = 'Error: y must not be None. x and z could.'

        obj = obj_res
        obj.df_res = obj.df_set.copy(deep=True)

        for k, v in iter(dic_req_fix.items()):
            if not isinstance(v, list):
                v = [v]

            assert k in obj.df_set.columns, msg1.format(k)
            if k is 'date':
                if 'range' in v:
                    v.remove('range')
                    mask = (obj.df_set[k] >= v[0]) & (obj.df_set[k] <= v[1])
                    obj.df_res = obj.df_res.loc[mask]
                else:
                    mask = (obj.df_set[k] == v[0]) & (obj.df_set[k] == v[1])
                    obj.df_res = obj.df_set.loc[mask]
            else:
                assert v[0] in obj.df_set[k].values, msg2.format(v[0], k)
                mask = (obj.df_set[k] == v[0])

                if len(v) > 1:
                    v.remove(v[0])
                    for v_ in v:
                        assert v_ in obj.df_set[k], msg2.format(v_, k)
                        mask = mask | (obj.df_set[k] == v_)
                obj.df_res = obj.df_set.loc[mask]

        self.df_slice = deepcopy(obj.df_res)

        if not isinstance(value, list):
            value = [value]

        assert y is not None, msg3

        li_xyz = [e for e in [x, y, z] if e is not None]

        if len(li_xyz) is 1:
            li_xz = []
        else:
            li_xz = deepcopy(li_xyz)
            li_xz.remove(y)

        values = value + li_xyz + list(dic_req_fix.keys())

        self.df_pivot = None
        tmp = self.df_slice[values]
        indexer = list(dic_req_fix.keys()) + li_xz
        if y_pos.lower() == 'index':
            if other_pos.lower() == 'column':
                self.df_pivot = tmp.pivot_table(value, index=y, columns=indexer)
            elif other_pos.lower() == 'index':
                self.df_pivot = tmp.set_index(indexer + [y] + value)
            else:
                raise Exception
        elif y_pos.lower() == 'column':
            if other_pos.lower() == 'column':
                self.df_pivot = tmp.set_index(indexer + [y] + value).T
            elif other_pos.lower() == 'index':
                self.df_pivot = tmp.pivot_table(value, index=indexer, columns=y)
            else:
                raise Exception
        else:
            raise Exception

        self.df_pivot = self.df_pivot.dropna()

    def save(self,
             folder_save='dump',
             name=None,
             tagged=True,
             excel=False):
        """
        """
        if name is None:
            name = 'SG_Research_ROTB'

        save_result(self.df_pivot,
                    folder_save,
                    name=name + '_Components_Slice',
                    tagged=tagged,
                    excel=excel)




# necessary to push to PyPI
# cf. http://peterdowns.com/posts/first-time-with-pypi.html
# cf. https://tom-christie.github.io/articles/pypi/
# cf. https://pythonhosted.org/setuptools/setuptools.html

# commands:
# one liner
# python setup.py sdist upload -r pypi
# 2 steps
# python setup.py sdist
# twine upload dist/*


from setuptools import setup, find_packages
from distutils.util import convert_path

packages = find_packages()
module = packages[0]

meta_ns = {}
ver_path = convert_path(module + '/__meta__.py')
with open(ver_path) as ver_file:
    exec(ver_file.read(), meta_ns)

name = meta_ns['__name__']
packages = packages
version = meta_ns['__version__']
description = meta_ns['__description__']
long_description = meta_ns['__long_description__']
author = meta_ns['__author__']
author_email = meta_ns['__author_email__']
url = meta_ns['__url__']
download_url = meta_ns['__download_url__']
keywords = meta_ns['__keywords__']
license = meta_ns['__license__']
classifiers = meta_ns['__classifiers__']
include_package_data = meta_ns['__include_package_data__']
package_data = meta_ns['__package_data__']
zip_safe = meta_ns['__zip_safe__']
entry_points = meta_ns['__entry_points__']

# read requirements.txt
with open('requirements.txt', 'r') as f:
    content = f.read()
li_req = content.split('\n')
install_requires = [e.strip() for e in li_req if len(e)]

# with open('README.rst') as f:
#     long_description = f.read()

setup(
    name=name,
    version=version,
    description=description,
    # long_description=long_description,
    author=author,
    author_email=author_email,
    url=url,
    download_url=download_url,
    keywords=keywords,
    license=license,
    classifiers=classifiers,
    packages=packages,
    install_requires=install_requires,
    include_package_data=include_package_data,
    package_data=package_data,
    zip_safe=zip_safe,
    entry_points=entry_points
)




import copy

import pandas as pd
import itertools as it
import operator

import sys


from timeit import default_timer as timer
from IPython.display import display, Markdown

from ._calendar import calendar as cal
from ._util import Util


LIMIT_SPLIT_REQUEST = 500


class RequestRotbComputeStrategyBase:
    """
    """

    def __init__(self,
                 dic=None,
                 date_fallback_next=True,
                 limit_split=LIMIT_SPLIT_REQUEST,
                 dic_url=None,
                 response_obj=None):
        """
        """
        self.response_obj = response_obj
        self.url = Util.build_url(dic_url)

        if dic:
            for k, v in dic.items():
                if not (isinstance(v, int) or isinstance(v, str)):
                    msg = 'param {} (={}) is not either a string or a number'
                    print(msg.format(k, v))
                setattr(self, k, v)
        self.dic_input = None
        self.dic_api = None
        self.li_dic_api = None
        self.df_top = None
        self.df_leg = None

        self.date_fallback_next = date_fallback_next
        self.limit_split = limit_split
        self.response_obj = response_obj
        self.leg_keys = [
            'curve',
            'expiry',
            'tenor',
            'strike',
            'nominal',
            'type',
            'settlement',
            'pricingStrategy',
            'strategyWeight',
            'customEntries',
        ]

    def _to_list(self, val):
        """
        """

        if isinstance(val, list):
            return val

        elif isinstance(val, int):
            return [val]
        elif ',' not in val:
            return [val]
        else:
            li_val = val.split(',')
            li_val = [e.strip() for e in li_val]
            return li_val

    def expand(self):
        """
        """
        dic_in_ = copy.copy(self.__dict__)
        dic_in_.pop('response_obj')
        dic_in = dic_in_

        for key in ['dic_input',
                    'dic_api',
                    'leg_keys',
                    'df_top',
                    'df_leg',
                    'li_dic_api',
                    'url',
                    'date_fallback_next',
                    'limit_split',
                    ]:
            dic_in.pop(key)

        # build dic as a prelude to dic_api
        self.dic_input = {}
        for k, v in dic_in.items():
            if k in self.leg_keys:
                # make params in leg to list
                self.dic_input[k] = self._to_list(v)
            else:
                # top params unchanged
                self.dic_input[k] = v

        # check dates are not holidays - if holiday fallback to next/previous
        li_date = self.dic_input.get('dates', None)
        if li_date:
            func = cal.get_BD_fallback_next if self.date_fallback_next else cal.get_BD_fallback_previous
            li_date_new = []
            dic_diff = {}
            for d in li_date:
                d_new = Util.date_to_str(func(d))
                li_date_new.append(d_new)
                if d_new != d:
                    dic_diff[d] = d_new
            if dic_diff:
                msg = '**<span style="color:red;">WARNING</span> -** These dates were modified as they are holidays:\n{}'
                msg = msg.format(str(dic_diff))
                display(Markdown(msg))
            self.dic_input['dates'] = li_date

        max_ = self.check_parameters()

        # create the list of dic
        li_leg = []
        for i in range(max_):
            dic = {k: v[i] for k, v in self.dic_input.items()
                   if k in self.leg_keys and v[i] is not "None"}
            li_leg.append(dic)

        # merge top params and params in legs expanded by cartesian product
        self.dic_api = {k: v for k, v in self.dic_input.items()
                        if not k in self.leg_keys}
        self.dic_api['swaptions'] = li_leg

        dic_top = {k: str(v) for k, v in self.dic_api.items()
                   if k != 'swaptions'}
        self.df_top = pd.DataFrame(dic_top, index=['Value']).T

        self.df_leg = pd.DataFrame(li_leg)

        # Alert on large number of requests - potential timeout
        n_leg = len(li_leg)

        li_date = self.dic_input.get('dates', None)
        if not li_date:
            try:
                li_date = Util.get_list_bday_between(self.dic_input['startDate'],
                                                     self.dic_input['endDate'])
            except Exception as e:
                msg = 'error: {}'.format(e.args[0])
                raise Exception(msg)

        n_date = len(li_date)
        n_total = n_leg * n_date

        msg = '**<span style="color:black;">INFO</span> -** Your request contains {} prices over {} dates i.e. total {} prices.'
        msg = msg.format(n_leg, n_date, n_total)
        display(Markdown(msg))

        if n_leg > self.limit_split:
            msg = '**<span style="color:red;">WARNING</span> -** Your request is probably too large. The API may timeout.\nIt is recommended to request up to ~{} prices per date.'
            msg = msg.format(self.limit_split)
            display(Markdown(msg))

        max_nb_date_per_chunk = max(1, int(1.0 * self.limit_split / n_leg))
        self.li_dic_api = self._split_input_by_dates(li_date,
                                                     max_nb_date_per_chunk,
                                                     li_leg)

    def check_parameters(self):
        """
        check and complete the input_dic if necessary
        """

        len_ = {k: len(v) for k, v in self.dic_input.items()
                if isinstance(v, list)}
        max_ = max(len_.items(), key=operator.itemgetter(1))[1]
        err_show = False

        if self.dic_input['weighting'] is 'DV01':
            if len(self.dic_input['nominal']) > 1:
                msg = '**<span style="color:red;">WARNING</span> -** Multiple nominal are not allowed when using DV01 weigthing by default only the nominal of the first leg will be used'
                display(Markdown(msg))

        for k, v in len_.items():
            if v is not max_:
                if not err_show:
                    err_show = True
                    msg = '**<span style="color:red;">WARNING</span> -**' \
                        + 'improper combinaison of parameters resulting in an unclear strategy' \
                        + ', remainder of the division of the longest parameters and others should be 0 '
                    assert (max_ % v) is 0, display(Markdown(msg))

                err = True
                self.dic_input[k] = self.dic_input[k] * int(max_ / v)

        if self.dic_input['weighting'] is 'DV01':
            if len(self.dic_input['nominal']) > 1:
                self.dic_input['nominal'] = [self.dic_input['nominal'][0]] + ['None'] * (max_ - 1)
        else:
            self.dic_input['nominal'] = [
                n * int(w) for n, w in zip(self.dic_input['nominal'],
                                           self.dic_input['strategyWeight'])
            ]
        if err:
            msg = '**<span style="color:red;">WARNING</span> -**'\
                + 'improper combinaison of parameters resulting in an unclear strategy'\
                + ', by default missing parameters will be filled by repetition'
            display(Markdown(msg))

        return max_

    def _split_input_by_dates(self,
                              li_date,
                              chunk_size,
                              li_leg):
        """
        """

        li_date_chunk = [li_date[i:i + chunk_size]
                         for i in range(0, len(li_date), chunk_size)]

        if len(li_date_chunk) > 1:
            msg = '**<span style="color:red;">WARNING</span> -** Your request will be split in {} chunks to avoid API timeout.'
            msg = msg.format(len(li_date_chunk))
            display(Markdown(msg))

        li_dic_api = []
        for date_chunk in li_date_chunk:
            dic_api_chunk = {k: v
                             for k, v in self.dic_input.items() if not k in self.leg_keys}

            # Special case: nominal only in first leg in weighting = DV01
            li_leg_chunk = copy.deepcopy(li_leg)
           # if self.dic_input['weighting'] == 'DV01':
            # for leg in li_leg_chunk[1:]:
            # print(leg)
            # leg.pop('nominal')

            # Build dic_api_chunk
            dic_api_chunk = {k: v
                             for k, v in self.dic_input.items() if not k in self.leg_keys}
            dic_api_chunk['swaptions'] = li_leg_chunk

            # Special case dates
            if 'startDate' in dic_api_chunk:
                dic_api_chunk['startDate'] = Util.date_to_str(date_chunk[0])
            if 'endDate' in dic_api_chunk:
                dic_api_chunk['endDate'] = Util.date_to_str(date_chunk[-1])
            # if 'dates' in dic_api_chunk:
            #     print(date_chunk)
            #     # dic_api_chunk['dates'] = [Util.date_to_str(d) for d in date_chunk]
            #     dic_api_chunk['dates'] = date_chunk
            #     print(dic_api_chunk['dates'])

            li_dic_api.append(dic_api_chunk)

        return li_dic_api

    def info(self):
        """
        """
        md = """
A RequestROTB object has the properties after the `expand()` method:
+ `url`: {}
+ `dic_input`: user input (dictionary)
+ `df_top`: parameters of the request not in a leg (dataframe)
+ `df_leg`: parameters of the request in a leg (dataframe)
    + A leg contains the following parameters: {}
    + Each param can have multiple comma separated values
+ `li_dic_api`: data ready to be sent to the API - built from user input by `expand()` (list of dictionaries)
    + This is a list because the request may be split in several chunks
    + Legs are constructed from user input and by filling missing values by repition if needed.
        """.format(self.url, str(self.leg_keys))
        return Markdown(md)

    def call_api(self,
                 api,
                 debug=False):
        """
        See https://analytics-api.sgmarkets.com/rotb/swagger/ui/index#!/Swaption/Swaption_StrategyComponents
        """
        t0 = timer()
        print('calling API...')
        li_raw_response = []
        for k, dic_api in enumerate(self.li_dic_api):

            raw_response = api.post(self.url,
                                    payload=dic_api)
            if debug:
                print('chunk {} raw response\n{}'.format(k, raw_response))
            li_raw_response.append(raw_response)
            t1 = timer()
            print('chunk {}/{} done in {:.2f} s'.format(1 + k,
                                                        len(self.li_dic_api),
                                                        t1 - t0))
        if debug:
            print('*** START DEBUG ***\n{}\n*** END DEBUG ***'.format(li_raw_response))
        response = self.response_obj(li_raw_data=li_raw_response,
                                     obj_req=self)
        return response





from IPython.display import Markdown

from .response_rotb_compute_strategy_components import ResponseRotbComputeStrategyComponents
from .response_rotb_compute_strategy_prices import ResponseRotbComputeStrategyPrices

from sgmarkets_api_auth.util import save_result


class ResponseRotbComputeStrategy:
    """
    TBU
    """

    def __init__(self,
                 li_raw_data=None,
                 obj_req=None):
        """
        """
        assert isinstance(li_raw_data, list), \
            'Error: li_raw_data must be a list - Run call_api() again with debug=True'
        for dic_res in li_raw_data:
            assert isinstance(dic_res, dict), \
                'Error: Each dic_res must be a list - Run call_api() again with debug=True'
            assert 'componentSeries' in dic_res, \
                'Error: componentSeries must be a key of each dic_res - Run call_api() again with debug=True'
            assert 'strategySerie' in dic_res, \
                'Error: stratedySerie must be a key of each dic_res - Run call_api() again with debug=True'

        self.components = ResponseRotbComputeStrategyComponents(li_raw_data, obj_req)
        self.strategy = ResponseRotbComputeStrategyPrices(li_raw_data, obj_req)

    def save(self,
             folder_save='dump',
             name=None,
             tagged=True,
             excel=False):
        """
        Split save for strategyprice type since no df.set in the object
        """
        if name is None:
            name = 'SG_Research_ROTB'

        save_result(self.components.df_set,
                    folder_save,
                    name=name + 'Components',
                    tagged=tagged,
                    excel=excel)

        save_result(self.strategy.df_res,
                    folder_save,
                    name=name + 'Strategy_res',
                    tagged=tagged,
                    excel=excel)

        save_result(self.strategy.df_req,
                    folder_save,
                    name=name + 'Strategy_req',
                    tagged=tagged,
                    excel=excel)

    def info(self):
        """
        """
        md = """
A PostprocessROTB object from ComputeStrategy endpoint has the properties:
    
+ Contains two sub-objects:
    +   `.components` : object of class computestrategycomponent   
    +    `.strategy`: object of class computestrategyprice
+ Common properties:
    + `df_req`: request data (dataframe)
    + `df_res`: response data (dataframe)
    + `dic_req_param`: params in request, each param contains a list of all values taken (dictionary)
    + `dic_res_param`: params in response, each param contains a list of all values taken (dictionary)
    + `raw_data`: raw data in response under key 'componentSeries' (dictionary)
+ Additional properties for .components sub-object:    
    + `df_set`: request and response data combined (dataframe)

and the methods:
+ `save()` to save the data as `.csv` and `.xlsx` files
        """
        return Markdown(md)
